﻿<DebuggerNonUserCode()> _
Friend NotInheritable Class UnsafeNativeMethods

#Region " Structures "

    <Runtime.InteropServices.StructLayout(Runtime.InteropServices.LayoutKind.Sequential)> _
    Public Structure KeyboardData
        Public vkCode As Integer
        Public scanCode As Integer
        Public flags As Integer
        Public time As Integer
        Public dwExtraInfo As IntPtr
    End Structure

    <Runtime.InteropServices.StructLayout(Runtime.InteropServices.LayoutKind.Sequential)> _
    Public Structure MouseData
        Public pt As Drawing.Point
        Public mouseData As Integer
        Public flags As Integer
        Public time As Integer
        Public dwExtraInfo As IntPtr
    End Structure

    <Runtime.InteropServices.StructLayout(Runtime.InteropServices.LayoutKind.Sequential)> _
    Public Structure MsInput
        Public dwType As Integer
        Public xi As MouseInput

        Public Shared Function GetSize() As Integer
            Return 4 + MouseInput.GetSize
        End Function
    End Structure

    <Runtime.InteropServices.StructLayout(Runtime.InteropServices.LayoutKind.Sequential)> _
    Public Structure MouseInput
        Public dx As Integer
        Public dy As Integer
        Public mouseData As Integer
        Public dwFlags As Integer
        Public time As Integer
        Public dwExtraInfo As IntPtr

        Public Shared Function GetSize() As Integer
            Return 20 + IntPtr.Size
        End Function
    End Structure

#End Region

#Region " Delegates "

    ''' <summary>
    ''' Represents the method that will handle keyboard messages.
    ''' </summary>
    Public Delegate Function WKCallBack(ByVal nCode As Integer, ByVal wParam As IntPtr, ByRef lParam As KeyboardData) As IntPtr

    ''' <summary>
    ''' Represents the method that will handle mouse messages.
    ''' </summary>
    Public Delegate Function WMCallBack(ByVal nCode As Integer, ByVal wParam As IntPtr, ByRef lParam As MouseData) As IntPtr

#End Region

#Region " Constants "

    Public Const HC_ACTION As Int32 = 0

#Region " Keyboard "

    Public Const WH_KEYBOARD As Int32 = 2
    Public Const WH_KEYBOARD_LL As Int32 = 13
    Public Const WM_KEYDOWN As Int32 = &H100
    Public Const WM_KEYUP As Int32 = &H101
    Public Const WM_SYSKEYDOWN As Int32 = &H104
    Public Const WM_SYSKEYUP As Int32 = &H105

#End Region

#Region " Mouse "

    Public Const WH_MOUSE As Int32 = 7
    Public Const WH_MOUSE_LL As Int32 = 14
    Public Const WM_MOUSEMOVE As Int32 = &H200
    Public Const WM_LBUTTONDOWN As Int32 = &H201
    Public Const WM_LBUTTONUP As Int32 = &H202
    Public Const WM_LBUTTONDBLCLK As Int32 = &H203
    Public Const WM_RBUTTONDOWN As Int32 = &H204
    Public Const WM_RBUTTONUP As Int32 = &H205
    Public Const WM_RBUTTONDBLCLK As Int32 = &H206
    Public Const WM_MBUTTONDOWN As Int32 = &H207
    Public Const WM_MBUTTONUP As Int32 = &H208
    Public Const WM_MBUTTONDBLCLK As Int32 = &H209
    Public Const WM_MOUSEWHEEL As Int32 = &H20A
    Public Const WM_MOUSEHWHEEL As Int32 = &H20E
    Public Const WM_XBUTTONDOWN As Int32 = &H20B
    Public Const WM_XBUTTONUP As Int32 = &H20C
    Public Const WM_XBUTTONDBLCLK As Int32 = &H20D
    Public Const WM_NCXBUTTONDOWN As Int32 = &HAB
    Public Const WM_NCXBUTTONUP As Int32 = &HAC
    Public Const WM_NCXBUTTONDBLCLK As Int32 = &HAD

    ' These are SynthesizeMouse constants
    Public Const MOUSEEVENTF_ABSOLUTE As Int32 = &H8000
    Public Const MOUSEEVENTF_LEFTDOWN As Int32 = &H2
    Public Const MOUSEEVENTF_LEFTUP As Int32 = &H4
    Public Const MOUSEEVENTF_MIDDLEDOWN As Int32 = &H20
    Public Const MOUSEEVENTF_MIDDLEUP As Int32 = &H40
    Public Const MOUSEEVENTF_MOVE As Int32 = &H1
    Public Const MOUSEEVENTF_RIGHTDOWN As Int32 = &H8
    Public Const MOUSEEVENTF_RIGHTUP As Int32 = &H10
    Public Const MOUSEEVENTF_VIRTUALDESK As Int32 = &H4000
    Public Const MOUSEEVENTF_WHEEL As Int32 = &H800
    Public Const MOUSEEVENTF_XDOWN As Int32 = &H80
    Public Const MOUSEEVENTF_XUP As Int32 = &H100
    Public Const XBUTTON1 As Int32 = &H1
    Public Const XBUTTON2 As Int32 = &H2
    Public Const WHEEL_DELTA As Int32 = 120

#End Region

#Region " Clipboard "

    Public Const WM_DRAWCLIPBOARD As Int32 = &H308
    Public Const WM_CHANGECBCHAIN As Int32 = &H30D

#End Region

#End Region

#Region " Methods "

    Private Sub New()
    End Sub

    <Runtime.InteropServices.DllImport("user32", SetLastError:=True)> _
    Public Overloads Shared Function SetWindowsHookEx( _
    ByVal idHook As Integer, _
    ByVal lpfn As WKCallBack, _
    ByVal hMod As IntPtr, _
    ByVal dwThreadId As Integer) As IntPtr
    End Function

    <Runtime.InteropServices.DllImport("user32", SetLastError:=True)> _
    Public Shared Function UnhookWindowsHookEx( _
    ByVal hook As IntPtr) As <Runtime.InteropServices.MarshalAs(Runtime.InteropServices.UnmanagedType.Bool)> Boolean
    End Function

    <Runtime.InteropServices.DllImport("user32")> _
    Public Overloads Shared Function CallNextHookEx( _
    ByVal hhk As IntPtr, _
    ByVal nCode As Integer, _
    ByVal wParam As IntPtr, _
    ByRef lParam As KeyboardData) As IntPtr
    End Function

    <Runtime.InteropServices.DllImport("user32", SetLastError:=True)> _
    Public Overloads Shared Function SetWindowsHookEx( _
    ByVal idHook As Integer, _
    ByVal lpfn As WMCallBack, _
    ByVal hMod As IntPtr, _
    ByVal dwThreadId As Integer) As IntPtr
    End Function

    <Runtime.InteropServices.DllImport("user32")> _
    Public Overloads Shared Function CallNextHookEx( _
    ByVal hhk As IntPtr, _
    ByVal nCode As Integer, _
    ByVal wParam As IntPtr, _
    ByRef lParam As MouseData) As IntPtr
    End Function

    <Runtime.InteropServices.DllImport("user32", SetLastError:=True)> _
    Public Shared Function SendInput( _
    ByVal cInputs As Integer, _
    ByRef pInputs As MsInput, _
    ByVal cbSize As Integer) As Integer
    End Function

    <Runtime.InteropServices.DllImport("user32")> _
    Public Shared Function WindowFromPoint( _
    ByVal pt As Drawing.Point) As IntPtr
    End Function

    <Runtime.InteropServices.DllImport("user32", SetLastError:=True)> _
    Public Shared Function SetClipboardViewer( _
    ByVal hWndNewViewer As IntPtr) As IntPtr
    End Function

    <Runtime.InteropServices.DllImport("user32")> _
    Public Shared Function ChangeClipboardChain( _
    ByVal hWndRemove As IntPtr, _
    ByVal hWndNewNext As IntPtr) As <Runtime.InteropServices.MarshalAs(Runtime.InteropServices.UnmanagedType.Bool)> Boolean
    End Function

#End Region

End Class
